# coding=utf-8
import telegram
from factory import Sequence, SubFactory, Factory
from factory.fuzzy import FuzzyText
from django.utils import timezone


class UserLibFactory(Factory):
    class Meta:
        model = telegram.User
    id = Sequence(lambda n: n+1)
    first_name = Sequence(lambda n: 'first_name_%d' % n)
    last_name = Sequence(lambda n: 'last_name_%d' % n)
    username = Sequence(lambda n: 'username_%d' % n)

class ChatLibFactory(Factory):
    class Meta:
        model = telegram.Chat
    id = Sequence(lambda n: n+1)
    type = "private"
    title = Sequence(lambda n: 'title_%d' % n)
    username = Sequence(lambda n: 'username_%d' % n)
    first_name = Sequence(lambda n: 'first_name_%d' % n)
    last_name = Sequence(lambda n: 'last_name_%d' % n)

class MessageLibFactory(Factory):
    class Meta:
        model = telegram.Message
    message_id = Sequence(lambda n: n+1)
    from_user = SubFactory(UserLibFactory)
    date = timezone.now()
    chat = SubFactory(ChatLibFactory)
    text = FuzzyText()    

class UpdateLibFactory(Factory):
    class Meta:
        model = telegram.Update
    update_id = Sequence(lambda n: n+1)
    message = SubFactory(MessageLibFactory)


# coding=utf-8
from factory import Sequence, DjangoModelFactory, PostGenerationMethodCall
from django.conf import settings

class UserWebFactory(DjangoModelFactory):
    username = Sequence(lambda n: 'user_name_%d' % n)
    email = Sequence(lambda n: 'mail_%s@example.com' % n)
    password = PostGenerationMethodCall('set_password', 'seed')
    is_active = True
    is_superuser = False

    class Meta:
        model = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


from django.conf.urls import url

def command(command, view):
    return url(r'^/%s(\s)*(?P<param>\w*)' % command, view)

def unknown_command(view):
    return url(r'^/(?P<unknown_command>\w+).*', view)

def regex(pattern, view):
    return url(pattern, view)

def message(view):
    return url(r'^(?P<message>.*)', view)


# -*- coding: utf-8 -*-
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from telegrambot.models import Chat
import logging
import os
import binascii
from django.utils.timezone import now
from datetime import timedelta

logger = logging.getLogger(__file__)


AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')

@python_2_unicode_compatible
class AuthToken(models.Model):
    key = models.CharField(max_length=40, primary_key=True)
    user = models.OneToOneField(AUTH_USER_MODEL, related_name='auth_token',
                                on_delete=models.CASCADE)
    chat_api = models.OneToOneField(Chat, related_name='auth_token',
                                    on_delete=models.CASCADE, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        verbose_name = _('Authentication Token')
        verbose_name_plural = _('Authentications Tokens')  

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super(AuthToken, self).save(*args, **kwargs)

    def generate_key(self):
        return binascii.hexlify(os.urandom(20)).decode()
    
    def expired(self):
        
        return self.created < now() - timedelta(hours=int(getattr(settings, 'TELEGRAM_BOT_TOKEN_EXPIRATION', '24')))

    def __str__(self):
        return self.key



from django.core.urlresolvers import RegexURLResolver
from django.core.urlresolvers import Resolver404

class HandlerNotFound(Exception):
    pass


class HandlerResolver(object):
    
    def __init__(self, conf):
        self.resolver = RegexURLResolver(r'^', conf)
        
    def resolve(self, update):
        try:
            resolver_match = self.resolver.resolve(update.message.text)
            return resolver_match
        except Resolver404:
            raise HandlerNotFound("No handler configured for  %s" % update.message.text)


# -*- coding: utf-8 -*-
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from telegram import Bot as BotAPI
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.core.urlresolvers import reverse
import logging
from telegrambot.models import User
from telegrambot.handlers import HandlerResolver
from telegrambot.handlers import HandlerNotFound

logger = logging.getLogger(__file__)


@python_2_unicode_compatible
class Bot(models.Model):
    token = models.CharField(_('Token'), max_length=100, db_index=True)
    user_api = models.OneToOneField(User, verbose_name=_("Bot User"), related_name='bot', 
                                    on_delete=models.CASCADE, blank=True, null=True)
    ssl_certificate = models.FileField(_("SSL certificate"), upload_to='telegrambot/ssl/', 
                                       blank=True, null=True)
    enabled = models.BooleanField(_('Enable'), default=True)
    created = models.DateTimeField(_('Date Created'), auto_now_add=True)
    modified = models.DateTimeField(_('Date Modified'), auto_now=True)    
    
    class Meta:
        verbose_name = _('Bot')
        verbose_name_plural = _('Bots')    
    
    def __init__(self, *args, **kwargs):
        super(Bot, self).__init__(*args, **kwargs)
        self._bot = None
        if self.token:
            self._bot = BotAPI(self.token)
            
    def __str__(self):
        return "%s" % (self.user_api.first_name or self.token if self.user_api else self.token)
            
    def handle(self, update):
        handlerconf = settings.TELEGRAM_BOT_HANDLERS_CONF
        resolver = HandlerResolver(handlerconf)
        try:
            resolver_match = resolver.resolve(update)
        except HandlerNotFound:
            logger.warning("Handler not found for %s" % update)
        else:
            callback, callback_args, callback_kwargs = resolver_match
            callback(self, update, **callback_kwargs)
        
    def send_message(self, chat_id, text, parse_mode=None, disable_web_page_preview=None, **kwargs):
        self._bot.sendMessage(chat_id=chat_id, text=text, parse_mode=parse_mode, 
                              disable_web_page_preview=disable_web_page_preview, **kwargs)        

@receiver(post_save, sender=Bot)
def set_api(sender, instance, **kwargs):
    #  set bot api if not yet
    if not instance._bot:
        instance._bot = BotAPI(instance.token)

    # set webhook
    url = None
    cert = None
    if instance.enabled:
        webhook = reverse('telegrambot:webhook', kwargs={'token': instance.token})        
        from django.contrib.sites.models import Site
        current_site = Site.objects.get_current()
        url = 'https://' + current_site.domain + webhook   
    if instance.ssl_certificate:
        cert = instance.ssl_certificate.open()
    instance._bot.setWebhook(webhook_url=url, 
                             certificate=cert)
    logger.info("Success: Webhook url %s for bot %s set" % (url, str(instance)))
    
    #  complete  Bot instance with api data
    if not instance.user_api:
        bot_api = instance._bot.getMe()
        user_api, _ = User.objects.get_or_create(**bot_api.to_dict())
        instance.user_api = user_api
        instance.save()
        logger.info("Success: Bot api info for bot %s set" % str(instance))



from functools import wraps
from telegrambot.models import Chat, Bot
from django.core.urlresolvers import reverse


def login_required(view_func):
    """
    Decorator for command views that checks that the chat is authenticated, 
    sends message with link for authenticated if necessary.
    """
    @wraps(view_func)
    def wrapper(bot, update, **kwargs):
        chat = Chat.objects.get(id=update.message.chat.id)
        if chat.is_authenticated():
            return view_func(bot, update, **kwargs)
        from telegrambot.bot_views.login import LoginBotView
        login_command_view = LoginBotView.as_command_view()
        bot_model = Bot.objects.get(token=bot.token)
        kwargs['link'] = reverse('telegrambot:auth', kwargs={'bot': bot_model.user_api.username}) 
        return login_command_view(bot, update, **kwargs)
    return wrapper


from telegrambot.handlers.conf import command  # noqa
from telegrambot.handlers.conf import message  # noqa
from telegrambot.handlers.conf import regex  # noqa
from telegrambot.handlers.conf import unknown_command  # noqa
from telegrambot.handlers.resolver import HandlerResolver, HandlerNotFound  # noqa